==> emodpy_covid-0.0.30-py3-none-any.whl: emodpy_covid/microstructure/change_ser_pop.py <==
import sys
import os
import json
import random
from collections import defaultdict
import math

# IDM code
from emod_api.serialization import SerializedPopulation
from . import hh_gen_toy
from . import add_hh_to_demo

risk_fn = None
def get_primary_sizes( total_pop ):
    # Note that people with "Community" age bucket will be assigned a community id in the range
    num_communities = 1
    # Pop/250
    num_elem_schools = math.ceil(total_pop/250)
    # Pop/1250
    num_jr_highs = math.ceil(total_pop/1250)
    # Pop/2500
    num_high_schools = math.ceil(total_pop/2500)
    # Pop/50
    num_workplaces = math.ceil(total_pop/50)
    return num_communities, num_elem_schools, num_jr_highs, num_high_schools, num_workplaces 

def count_dict( map2lsit ):
    accum = 0
    for key in map2lsit.keys():
        accum += len(map2lsit[key])
    return accum

def map_age_bucket_to_primary( total_pop ): 
    communities, es_es, jh_es, hs_es, works = get_primary_sizes( total_pop )

    age_bucket_2_primary_id = {}
    age_bucket_2_primary_id["Community"] = {}
    age_bucket_2_primary_id["Community"]["min"] = 0
    age_bucket_2_primary_id["Community"]["range"] = communities 
    age_bucket_2_primary_id["ES"] = {}
    age_bucket_2_primary_id["ES"]["min"] = age_bucket_2_primary_id["Community"]["min"] + age_bucket_2_primary_id["Community"]["range"] # 100
    age_bucket_2_primary_id["ES"]["range"] = es_es
    age_bucket_2_primary_id["JH"] = {}
    age_bucket_2_primary_id["JH"]["min"] = age_bucket_2_primary_id["ES"]["min"] + age_bucket_2_primary_id["ES"]["range"] # 120
    age_bucket_2_primary_id["JH"]["range"] = jh_es
    age_bucket_2_primary_id["HS"] = {}
    age_bucket_2_primary_id["HS"]["min"] = age_bucket_2_primary_id["JH"]["min"] + age_bucket_2_primary_id["JH"]["range"] # 140
    age_bucket_2_primary_id["HS"]["range"] = hs_es
    age_bucket_2_primary_id["Work"] = {}
    age_bucket_2_primary_id["Work"]["min"] = age_bucket_2_primary_id["HS"]["min"] + age_bucket_2_primary_id["HS"]["range"] # 160
    age_bucket_2_primary_id["Work"]["range"] = works
    return age_bucket_2_primary_id 

def change_ser_pop():
    path = sys.argv[1]
    if not os.path.exists( path ):
        print( f"Couldn't find specified serialized population file: {path}." )
        sys.exit(0)

    ser_pop = SerializedPopulation.SerializedPopulation(path) 

    node_0 = ser_pop.nodes[0]
    pop_size = len( node_0["individualHumans"] )
    households_filename = hh_gen_toy.write_households( pop_size )

    """
    foreach household as hh_id...
    foreach person in hh_id as [ age_bucket ]....
    search through node_0["individualHumans"] for unique individual eq age_bucket in [idx]["Properties"]
    node_0["individualHumans][idx]["Properties"].append( "Household:<hh_id> )

    save this.

    """
    # poc, no serious households
    if not os.path.exists( households_filename ):
        print( f"Couldn't find {households_filename} file. Need that to define household structure." )
        sys.exit()

    hh = json.loads( open( households_filename ).read() ) # this is a household structure
    agegroup_to_hh_map = defaultdict(list)
    primary_2_ind_map = defaultdict(list)

    # Agegroup_to_hh_map will be indexed by "W", "C", "ES", "JH", "HS
    # And each element will contain a list of household ids with at least one of these.
    # If there are multiple of a group in household, you'll get duplicates of the form
    # "W" : [114, 115, 115, 116] <- household 115 has two "W"s
    for hh_id in hh:
        for elem in hh[hh_id]:
            agegroup_to_hh_map[ elem ].append( int(hh_id) )

    age_bucket_2_primary_id = map_age_bucket_to_primary( pop_size )

    hh_id_set = []
    primary_id_set = []
    for person in range(len(node_0["individualHumans"])):
        #if person < 2:
            #continue
        hh_id = -1
        # 1) Figure out what age bucket this person is in.
        age_bucket = node_0["individualHumans"][person]["Properties"][0].split(':')[1]
        # 2) Get range of ids that cover this bucket
        newmin = age_bucket_2_primary_id[age_bucket]["min"]
        newmax = newmin + age_bucket_2_primary_id[age_bucket]["range"]
        print( f"newmin = {newmin}, newmax = {newmax}." )
        # 3) Get primary_id as random number in this range
        primary_id = random.randint( newmin, newmax-1 )
        # 4) Do the assignment
        #primary_id = person
        print( f"Assigning individual {person} to primary group {primary_id} ." )
        node_0["individualHumans"][person]["Properties"][0] = "Primary:" + str(primary_id)
        primary_2_ind_map[primary_id].append( person )
        # 5) Store every used primary id to report to DTK.
        if primary_id not in primary_id_set:
            primary_id_set.append( primary_id )
        # age_bucket is one of: Communnity, Work, ES, JH, or HS.
        # we want to change this to a value Primary_XXX but XXX should be based on their age_bucket

        # 6) Search through household structure for next unfilled slot matching that 
        if len( agegroup_to_hh_map[ age_bucket ] ) > 0:
            hh_id = agegroup_to_hh_map[ age_bucket ].pop() 
        else:
            for agebucketkey in agegroup_to_hh_map.keys():
                if len( agegroup_to_hh_map[ agebucketkey ] ) > 0:
                    hh_id = agegroup_to_hh_map[ agebucketkey ].pop() 
                    break
        if hh_id not in hh_id_set:
            hh_id_set.append( hh_id )

        #hh_id = random.randint(0,50) # 10 works, 12, 15 & 25 gives warnins, no crash; 50 fails
        print( "Assigning individual {} to household id {} leaving {} elements.".format( person, hh_id, count_dict(agegroup_to_hh_map) ) )

        # First, remove any existing Household property
        for ip in node_0["individualHumans"][person]["Properties"]:
            if "Household" in ip:
                print( "Removing IP " + str( ip ) )
                node_0["individualHumans"][person]["Properties"].remove( ip )

        node_0["individualHumans"][person]["Properties"].append( "Household:" + str(hh_id) )
        if hh_id == -1:
            print( "Failed to find household id for " + str( person ) )
  
        if risk_fn:
            node_0["individualHumans"][person]["m_is_highrisk"] = risk_fn( node_0["individualHumans"][person] )

    save_file_path = input( "Enter filename of new serialized population (e.g., my_sp_file.dtk): " ) 
    ser_pop.write( save_file_path )

    hh_id_filename = "household_ids.json"
    print( f"Writing file {hh_id_filename}." )
    with open( hh_id_filename , "w" ) as new_hhds_json:
        json.dump( sorted(hh_id_set), new_hhds_json )

    primary_id_filename = "primary_ids.json"
    print( f"Writing file {primary_id_filename}." )
    with open( primary_id_filename , "w" ) as new_primaryids_json:
        json.dump( sorted(primary_id_set), new_primaryids_json )

    with open( "primaries.json", "w" ) as primaries_json:
        json.dump( primary_2_ind_map, primaries_json, sort_keys=True, indent=4 )
        
    #TBD: Prompt user for input_demog, output_demog
    input_demog  = sys.argv[2]
    output_demog = input( "Enter path/filename of new demographics file: " )
    add_hh_to_demo.doit( primary_id_filename, hh_id_filename, input_demog, output_demog  )


if __name__ == "__main__":

    # prompt user for population size
    #pop_size = int( input( "Enter Population Size: " ) ) 

    """
    Inputs:
    This script looks for a households.json file, which explicitly contains a dict of household ids and the
    constituents of those households, but only but age bucket (e.g., Worker, Elementary Schooler, Community-Person).
    It also takes a .dtk input file.

    Outputs:
    - my_sp_file.dtk
    - household_ids.json
    - primary_ids.json

    Function:
    - Primary ID ends up as 0th IP
    - Household ID ends up as 1st IP
    """
    change_ser_pop()

==> emodpy_covid-0.0.32-py3-none-any.whl: emodpy_covid/microstructure/change_ser_pop.py <==
import sys
import os
import json
import random
from collections import defaultdict
import math
import argparse

# IDM code
from emod_api.serialization import SerializedPopulation
from . import hh_gen_toy
from . import add_hh_to_demo

risk_fn = None
def get_primary_sizes( total_pop ):
    # Note that people with "Community" age bucket will be assigned a community id in the range
    num_communities = 1
    # Pop/250
    num_elem_schools = math.ceil(total_pop/250)
    # Pop/1250
    num_jr_highs = math.ceil(total_pop/1250)
    # Pop/2500
    num_high_schools = math.ceil(total_pop/2500)
    # Pop/50
    num_workplaces = math.ceil(total_pop/50)
    return num_communities, num_elem_schools, num_jr_highs, num_high_schools, num_workplaces 

def count_dict( map2lsit ):
    accum = 0
    for key in map2lsit.keys():
        accum += len(map2lsit[key])
    return accum

def map_age_bucket_to_primary( total_pop ): 
    communities, es_es, jh_es, hs_es, works = get_primary_sizes( total_pop )

    age_bucket_2_primary_id = {}
    age_bucket_2_primary_id["Community"] = {}
    age_bucket_2_primary_id["Community"]["min"] = 0
    age_bucket_2_primary_id["Community"]["range"] = communities 
    age_bucket_2_primary_id["ES"] = {}
    age_bucket_2_primary_id["ES"]["min"] = age_bucket_2_primary_id["Community"]["min"] + age_bucket_2_primary_id["Community"]["range"] # 100
    age_bucket_2_primary_id["ES"]["range"] = es_es
    age_bucket_2_primary_id["JH"] = {}
    age_bucket_2_primary_id["JH"]["min"] = age_bucket_2_primary_id["ES"]["min"] + age_bucket_2_primary_id["ES"]["range"] # 120
    age_bucket_2_primary_id["JH"]["range"] = jh_es
    age_bucket_2_primary_id["HS"] = {}
    age_bucket_2_primary_id["HS"]["min"] = age_bucket_2_primary_id["JH"]["min"] + age_bucket_2_primary_id["JH"]["range"] # 140
    age_bucket_2_primary_id["HS"]["range"] = hs_es
    age_bucket_2_primary_id["Work"] = {}
    age_bucket_2_primary_id["Work"]["min"] = age_bucket_2_primary_id["HS"]["min"] + age_bucket_2_primary_id["HS"]["range"] # 160
    age_bucket_2_primary_id["Work"]["range"] = works
    return age_bucket_2_primary_id 

def change_ser_pop(path, input_demog, save_file_path=None, output_demog=None):
    print( f"change_ser_pop called with 'path'={path}, 'input_demog'={input_demog}, 'save_file_path'={save_file_path}, 'output_demog'={output_demog}." )

    if not os.path.exists( path ):
        print( f"Couldn't find specified serialized population file: {path}." )
        sys.exit(0)

    ser_pop = SerializedPopulation.SerializedPopulation(path) 

    node_0 = ser_pop.nodes[0]
    pop_size = len( node_0["individualHumans"] )
    households_filename = hh_gen_toy.write_households( pop_size )

    """
    foreach household as hh_id...
    foreach person in hh_id as [ age_bucket ]....
    search through node_0["individualHumans"] for unique individual eq age_bucket in [idx]["Properties"]
    node_0["individualHumans][idx]["Properties"].append( "Household:<hh_id> )

    save this.

    """
    # poc, no serious households
    if not os.path.exists( households_filename ):
        print( f"Couldn't find {households_filename} file. Need that to define household structure." )
        sys.exit()

    hh = json.loads( open( households_filename ).read() ) # this is a household structure
    agegroup_to_hh_map = defaultdict(list)
    primary_2_ind_map = defaultdict(list)

    # Agegroup_to_hh_map will be indexed by "W", "C", "ES", "JH", "HS
    # And each element will contain a list of household ids with at least one of these.
    # If there are multiple of a group in household, you'll get duplicates of the form
    # "W" : [114, 115, 115, 116] <- household 115 has two "W"s
    for hh_id in hh:
        for elem in hh[hh_id]:
            agegroup_to_hh_map[ elem ].append( int(hh_id) )

    age_bucket_2_primary_id = map_age_bucket_to_primary( pop_size )

    hh_id_set = []
    primary_id_set = []
    for person in range(len(node_0["individualHumans"])):
        #if person < 2:
            #continue
        hh_id = -1
        # 1) Figure out what age bucket this person is in.
        if "individualHumans" not in node_0:
            print( "ERROR: Failed to find 'individualHumans' in node_0 serialized population input." )
            continue
        if "Properties" not in node_0["individualHumans"][person]:
            print( f"ERROR: Failed to find {person} in node_0['individualHumans'][{person}] serialized population input." )
            continue
        if len(node_0["individualHumans"][person]["Properties"])==0:
            print( f"ERROR: Failed to find any entries in node_0['individualHumans'][{person}] serialized population input." )
            continue
        if person == 0 and len(node_0["individualHumans"][person]["Properties"][0].split(':'))<=1:
            print( f"ERROR: Failed to find enough entries in node_0['individualHumans'][{person}][0].split(':') serialized population input." )
            continue
        age_bucket = node_0["individualHumans"][person]["Properties"][0].split(':')[1]
        # 2) Get range of ids that cover this bucket
        newmin = age_bucket_2_primary_id[age_bucket]["min"]
        newmax = newmin + age_bucket_2_primary_id[age_bucket]["range"]
        print( f"newmin = {newmin}, newmax = {newmax}." )
        # 3) Get primary_id as random number in this range
        primary_id = random.randint( newmin, newmax-1 )
        # 4) Do the assignment
        #primary_id = person
        print( f"Assigning individual {person} to primary group {primary_id} ." )
        node_0["individualHumans"][person]["Properties"][0] = "Primary:" + str(primary_id)
        primary_2_ind_map[primary_id].append( person )
        # 5) Store every used primary id to report to DTK.
        if primary_id not in primary_id_set:
            primary_id_set.append( primary_id )
        # age_bucket is one of: Communnity, Work, ES, JH, or HS.
        # we want to change this to a value Primary_XXX but XXX should be based on their age_bucket

        # 6) Search through household structure for next unfilled slot matching that 
        if len( agegroup_to_hh_map[ age_bucket ] ) > 0:
            hh_id = agegroup_to_hh_map[ age_bucket ].pop() 
        else:
            for agebucketkey in agegroup_to_hh_map.keys():
                if len( agegroup_to_hh_map[ agebucketkey ] ) > 0:
                    hh_id = agegroup_to_hh_map[ agebucketkey ].pop() 
                    break
        if hh_id not in hh_id_set:
            hh_id_set.append( hh_id )

        #hh_id = random.randint(0,50) # 10 works, 12, 15 & 25 gives warnins, no crash; 50 fails
        print( "Assigning individual {} to household id {} leaving {} elements.".format( person, hh_id, count_dict(agegroup_to_hh_map) ) )

        # First, remove any existing Household property
        for ip in node_0["individualHumans"][person]["Properties"]:
            if "Household" in ip:
                print( "Removing IP " + str( ip ) )
                node_0["individualHumans"][person]["Properties"].remove( ip )

        node_0["individualHumans"][person]["Properties"].append( "Household:" + str(hh_id) )
        if hh_id == -1:
            print( "Failed to find household id for " + str( person ) )
  
        if risk_fn:
            node_0["individualHumans"][person]["m_is_highrisk"] = risk_fn( node_0["individualHumans"][person] )

    if save_file_path==None:
        save_file_path = input( "Enter filename of new serialized population (e.g., my_sp_file.dtk): " ) 
    ser_pop.write( save_file_path )

    hh_id_filename = "household_ids.json"
    print( f"Writing file {hh_id_filename}." )
    with open( hh_id_filename , "w" ) as new_hhds_json:
        json.dump( sorted(hh_id_set), new_hhds_json )

    primary_id_filename = "primary_ids.json"
    print( f"Writing file {primary_id_filename}." )
    with open( primary_id_filename , "w" ) as new_primaryids_json:
        json.dump( sorted(primary_id_set), new_primaryids_json )

    with open( "primaries.json", "w" ) as primaries_json:
        json.dump( primary_2_ind_map, primaries_json, sort_keys=True, indent=4 )
        
    if output_demog == None:
        output_demog = input( "Enter path/filename of new demographics file: " )
    add_hh_to_demo.doit( primary_id_filename, hh_id_filename, input_demog, output_demog  )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-si', '--serialized_pop_input', type=str, help="/path/to/serialized_population_file_from_stage1.dtk")
    parser.add_argument('-di', '--demographic_input', type=str, help="/path/to/demographics_input.json")
    parser.add_argument('-so', '--serialized_pop_output', type=str, default=None, help="/path/to/serialized_population_microstructure_file.dtk")
    parser.add_argument('-do', '--demographic_output', type=str, default=None, help="/path/to/demographics_output.json")
    args = parser.parse_args() 

    # prompt user for population size
    #pop_size = int( input( "Enter Population Size: " ) ) 

    """
    Inputs:
    This script looks for a households.json file, which explicitly contains a dict of household ids and the
    constituents of those households, but only but age bucket (e.g., Worker, Elementary Schooler, Community-Person).
    It also takes a .dtk input file.

    Outputs:
    - my_sp_file.dtk
    - household_ids.json
    - primary_ids.json

    Function:
    - Primary ID ends up as 0th IP
    - Household ID ends up as 1st IP
    """
    change_ser_pop(args.serialized_pop_input, args.demographic_input, args.serialized_pop_output, args.demographic_output)
